
# q-expansion of newform 294.4.e.j, downloaded from the LMFDB on 08 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 294
weight = 4
poly_data = [1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[197, [1, 0]], [199, [0, -1]]]
aps_data = [[0, 2], [3, -3], [0, 8], [0, 0], [-40, 40], [-4, 0], [-84, 84], [0, 148], [0, -84], [58, 0], [-136, 136], [0, 222], [-420, 0], [-164, 0], [0, 488], [-478, 478], [548, -548], [0, 692], [908, -908], [-524, 0], [440, -440], [0, -1216], [684, 0], [0, 604], [832, 0], [464, -464], [0, -632], [0, 160], [2198, -2198], [770, 0], [-184, 0], [0, -1452], [-646, 646], [3012, 0], [0, 3170], [1880, -1880], [604, -604], [0, -1116], [1784, 0], [0, -344], [-1392, 1392], [-4052, 0], [0, 3108], [-50, 50], [-162, 0], [1544, -1544], [-1204, 0], [-2000, 0], [388, -388], [0, 4180], [0, 1322], [2412, 0], [-4336, 4336], [-764, 0], [0, 4300], [3860, -3860], [-2800, 2800], [0, -4880], [6674, -6674], [-9402, 0], [-9100, 9100], [-5952, 0], [3004, 0], [688, -688], [0, 5592], [0, 2922], [0, 7492], [10766, 0], [3984, -3984], [180, 0], [-10428, 10428], [0, -8684], [5648, -5648], [0, 2546], [8268, 0], [0, -10872], [-10434, 10434], [0, -3044], [0, -8910], [5616, -5616], [8932, 0], [-5538, 0], [6700, -6700], [5048, 0], [0, -1344], [0, 4392], [3666, 0], [0, -26], [-7656, 0], [12608, 0], [0, -3068], [6456, -6456], [-11896, 11896], [-264, 0], [0, 2628], [13568, 0], [0, 20656], [3628, -3628], [0, 4852], [0, 7130], [-12788, 0], [-2406, 2406], [-25412, 25412], [0, 9690], [-5604, 5604], [-21568, 21568], [-20300, 0], [0, 13812], [21996, -21996], [-8368, 0], [0, 21504], [10270, -10270], [28358, 0], [16292, -16292], [11256, 0], [-15518, 15518], [10452, 0], [72, -72], [0, -11962], [-6016, 0], [26068, -26068], [-20530, 0], [0, -10056], [-6152, 6152], [0, -14716], [28202, 0], [0, -22114], [0, -9288], [-23848, 0], [0, -34756], [-26044, 26044], [36204, 0], [0, 11424], [-16622, 0], [0, -38524], [18440, 0], [-13968, 13968], [10916, -10916], [-12360, 0], [-3402, 3402], [-292, 0], [0, -6910], [-568, 568], [-12144, 0], [14828, -14828], [22824, 0], [41780, 0], [-21420, 21420], [0, -18132], [0, -24036], [0, 4374], [46348, 0], [-20660, 0], [0, -1800], [41996, -41996], [-41308, 0], [0, -3936], [0, -7212], [38976, 0], [53544, -53544], [0, 21392], [21162, 0], [8224, 0], [0, -8140], [-32158, 32158], [-41416, 41416], [-12296, 12296], [57652, -57652]]
